@@ -46,9 +46,6 @@ unused_docs = [
     'library/xml.etree',
 ]
 
-# Ignore .rst in Sphinx its self.
-exclude_trees = ['tools/sphinx']
-
 # Relative filename of the reference count data file.
 refcount_file = 'data/refcounts.dat'
 
@@ -59,6 +56,9 @@ add_function_parentheses = True
 # unit titles (such as .. function::).
 add_module_names = True
 
+# By default, highlight as Python 3.
+highlight_language = 'python3'
+
 
 # Options for HTML output
 # -----------------------
@@ -83,7 +83,7 @@ html_additional_pages = {
 }
 
 # Output an OpenSearch description file.
-html_use_opensearch = 'http://docs.python.org/dev'
+html_use_opensearch = 'http://docs.python.org/dev/3.0'
 
 # Additional static files.
 html_static_path = ['tools/sphinxext/static']
@@ -125,9 +125,7 @@ latex_documents = [
     ('tutorial/index', 'tutorial.tex',
      'Python Tutorial', _stdauthor, 'manual'),
     ('using/index', 'using.tex',
-     'Python Setup and Usage', _stdauthor, 'manual'),
-    ('faq/index', 'faq.tex',
-     'Python Frequently Asked Questions', _stdauthor, 'manual'),
+     'Using Python', _stdauthor, 'manual'),
     ('whatsnew/' + version, 'whatsnew.tex',
      'What\'s New in Python', 'A. M. Kuchling', 'howto'),
 ]
@@ -150,9 +148,6 @@ latex_preamble = r'''
 # Documents to append as an appendix to all manuals.
 latex_appendices = ['glossary', 'about', 'license', 'copyright']
 
-# Get LaTeX to handle Unicode correctly
-latex_elements = {'inputenc': r'\usepackage[utf8x]{inputenc}'}
-
 # Options for the coverage checker
 # --------------------------------
 
