@@ -9,27 +9,6 @@ else:
     THIS_ENDIAN = ">"
     OTHER_ENDIAN = "<"
 
-class memoryview(object):
-    # This class creates a memoryview - like object from data returned
-    # by the private _ctypes._buffer_info() function, just enough for
-    # these tests.
-    #
-    # It can be removed when the py3k memoryview object is backported.
-    def __init__(self, ob):
-        from _ctypes import _buffer_info
-        self.format, self.ndim, self.shape = _buffer_info(ob)
-        if self.shape == ():
-            self.shape = None
-            self.itemsize = sizeof(ob)
-        else:
-            size = sizeof(ob)
-            for dim in self.shape:
-                size /= dim
-            self.itemsize = size
-        self.strides = None
-        self.readonly = False
-        self.size = sizeof(ob)
-
 def normalize(format):
     # Remove current endian specifier and white space from a format
     # string
@@ -46,7 +25,10 @@ class Test(unittest.TestCase):
             v = memoryview(ob)
             try:
                 self.failUnlessEqual(normalize(v.format), normalize(fmt))
-                self.failUnlessEqual(v.size, sizeof(ob))
+                if shape is not None:
+                    self.failUnlessEqual(len(v), shape[0])
+                else:
+                    self.failUnlessEqual(len(v) * sizeof(itemtp), sizeof(ob))
                 self.failUnlessEqual(v.itemsize, sizeof(itemtp))
                 self.failUnlessEqual(v.shape, shape)
                 # ctypes object always have a non-strided memory block
@@ -58,7 +40,7 @@ class Test(unittest.TestCase):
                     n = 1
                     for dim in v.shape:
                         n = n * dim
-                    self.failUnlessEqual(v.itemsize * n, v.size)
+                    self.failUnlessEqual(n * v.itemsize, len(v.tobytes()))
             except:
                 # so that we can see the failing type
                 print(tp)
@@ -70,7 +52,10 @@ class Test(unittest.TestCase):
             v = memoryview(ob)
             try:
                 self.failUnlessEqual(v.format, fmt)
-                self.failUnlessEqual(v.size, sizeof(ob))
+                if shape is not None:
+                    self.failUnlessEqual(len(v), shape[0])
+                else:
+                    self.failUnlessEqual(len(v) * sizeof(itemtp), sizeof(ob))
                 self.failUnlessEqual(v.itemsize, sizeof(itemtp))
                 self.failUnlessEqual(v.shape, shape)
                 # ctypes object always have a non-strided memory block
@@ -82,7 +67,7 @@ class Test(unittest.TestCase):
                     n = 1
                     for dim in v.shape:
                         n = n * dim
-                    self.failUnlessEqual(v.itemsize * n, v.size)
+                    self.failUnlessEqual(n, len(v))
             except:
                 # so that we can see the failing type
                 print(tp)
