@@ -34,6 +34,9 @@ static int
 memory_getbuf(PyMemoryViewObject *self, Py_buffer *view, int flags)
 {
     int res = 0;
+    /* XXX for whatever reason fixing the flags seems necessary */
+    if (self->view.readonly)
+        flags &= ~PyBUF_WRITABLE;
     if (self->view.obj != NULL)
         res = PyObject_GetBuffer(self->view.obj, view, flags);
     if (view)
@@ -73,7 +76,6 @@ PyObject *
 PyMemoryView_FromObject(PyObject *base)
 {
     PyMemoryViewObject *mview;
-    Py_buffer view;
 
     if (!PyObject_CheckBuffer(base)) {
         PyErr_SetString(PyExc_TypeError,
@@ -82,17 +84,20 @@ PyMemoryView_FromObject(PyObject *base)
         return NULL;
     }
 
-    if (PyObject_GetBuffer(base, &view, PyBUF_FULL_RO) < 0)
+    mview = (PyMemoryViewObject *)
+        PyObject_GC_New(PyMemoryViewObject, &PyMemoryView_Type);
+    if (mview == NULL)
         return NULL;
 
-    mview = (PyMemoryViewObject *)PyMemoryView_FromBuffer(&view);
-    if (mview == NULL) {
-        PyBuffer_Release(&view);
+    mview->base = NULL;
+    if (PyObject_GetBuffer(base, &(mview->view), PyBUF_FULL_RO) < 0) {
+        Py_DECREF(mview);
         return NULL;
     }
 
     mview->base = base;
     Py_INCREF(base);
+    _PyObject_GC_TRACK(mview);
     return (PyObject *)mview;
 }
 
@@ -169,6 +174,9 @@ _strided_copy_nd(char *dest, char *src, int nd, Py_ssize_t *shape,
     return;
 }
 
+void _add_one_to_index_F(int nd, Py_ssize_t *index, Py_ssize_t *shape);
+void _add_one_to_index_C(int nd, Py_ssize_t *index, Py_ssize_t *shape);
+
 static int
 _indirect_copy_nd(char *dest, Py_buffer *view, char fort)
 {
@@ -176,7 +184,7 @@ _indirect_copy_nd(char *dest, Py_buffer *view, char fort)
     int k;
     Py_ssize_t elements;
     char *ptr;
-    void (*func)(int, Py_ssize_t *, const Py_ssize_t *);
+    void (*func)(int, Py_ssize_t *, Py_ssize_t *);
 
     if (view->ndim > PY_SSIZE_T_MAX / sizeof(Py_ssize_t)) {
         PyErr_NoMemory();
@@ -197,10 +205,10 @@ _indirect_copy_nd(char *dest, Py_buffer *view, char fort)
         elements *= view->shape[k];
     }
     if (fort == 'F') {
-        func = _Py_add_one_to_index_F;
+        func = _add_one_to_index_F;
     }
     else {
-        func = _Py_add_one_to_index_C;
+        func = _add_one_to_index_C;
     }
     while (elements--) {
         func(view->ndim, indices, view->shape);
@@ -327,7 +335,7 @@ PyMemoryView_GetContiguous(PyObject *obj, int buffertype, char fort)
 static PyObject *
 memory_format_get(PyMemoryViewObject *self)
 {
-    return PyString_FromString(self->view.format);
+    return PyUnicode_FromString(self->view.format);
 }
 
 static PyObject *
@@ -391,7 +399,7 @@ memory_ndim_get(PyMemoryViewObject *self)
 }
 
 static PyGetSetDef memory_getsetlist[] ={
-    {"format",          (getter)memory_format_get,      NULL, NULL},
+    {"format",                (getter)memory_format_get,      NULL, NULL},
     {"itemsize",        (getter)memory_itemsize_get,    NULL, NULL},
     {"shape",           (getter)memory_shape_get,       NULL, NULL},
     {"strides",         (getter)memory_strides_get,     NULL, NULL},
@@ -403,18 +411,10 @@ static PyGetSetDef memory_getsetlist[] ={
 
 
 static PyObject *
-memory_tobytes(PyMemoryViewObject *self, PyObject *noargs)
+memory_tobytes(PyMemoryViewObject *mem, PyObject *noargs)
 {
-    Py_buffer view;
-    PyObject *res;
-
-    if (PyObject_GetBuffer((PyObject *)self, &view, PyBUF_SIMPLE) < 0)
-        return NULL;
-
-    res = PyBytes_FromStringAndSize(NULL, view.len);
-    PyBuffer_ToContiguous(PyBytes_AS_STRING(res), &view, view.len, 'C');
-    PyBuffer_Release(&view);
-    return res;
+    return PyObject_CallFunctionObjArgs(
+            (PyObject *) &PyBytes_Type, mem, NULL);
 }
 
 /* TODO: rewrite this function using the struct module to unpack
@@ -443,7 +443,7 @@ memory_tolist(PyMemoryViewObject *mem, PyObject *noargs)
         return NULL;
     buf = view->buf;
     for (i = 0; i < view->len; i++) {
-        item = PyInt_FromLong((unsigned char) *buf);
+        item = PyLong_FromUnsignedLong((unsigned char) *buf);
         if (item == NULL) {
             Py_DECREF(res);
             return NULL;
@@ -495,7 +495,7 @@ memory_dealloc(PyMemoryViewObject *self)
 static PyObject *
 memory_repr(PyMemoryViewObject *self)
 {
-    return PyString_FromFormat("<memory at %p>", self);
+    return PyUnicode_FromFormat("<memory at %p>", self);
 }
 
 /* Sequence methods */
@@ -505,49 +505,6 @@ memory_length(PyMemoryViewObject *self)
     return get_shape0(&self->view);
 }
 
-/* Alternate version of memory_subcript that only accepts indices.
-   Used by PySeqIter_New().
-*/
-static PyObject *
-memory_item(PyMemoryViewObject *self, Py_ssize_t result)
-{
-    Py_buffer *view = &(self->view);
-
-    if (view->ndim == 0) {
-        PyErr_SetString(PyExc_IndexError,
-                        "invalid indexing of 0-dim memory");
-        return NULL;
-    }
-    if (view->ndim == 1) {
-        /* Return a bytes object */
-        char *ptr;
-        ptr = (char *)view->buf;
-        if (result < 0) {
-            result += get_shape0(view);
-        }
-        if ((result < 0) || (result >= get_shape0(view))) {
-            PyErr_SetString(PyExc_IndexError,
-                                "index out of bounds");
-            return NULL;
-        }
-        if (view->strides == NULL)
-            ptr += view->itemsize * result;
-        else
-            ptr += view->strides[0] * result;
-        if (view->suboffsets != NULL &&
-            view->suboffsets[0] >= 0) {
-            ptr = *((char **)ptr) + view->suboffsets[0];
-        }
-        return PyBytes_FromStringAndSize(ptr, view->itemsize);
-    } else {
-        /* Return a new memory-view object */
-        Py_buffer newview;
-        memset(&newview, 0, sizeof(newview));
-        /* XXX:  This needs to be fixed so it actually returns a sub-view */
-        return PyMemoryView_FromBuffer(&newview);
-    }
-}
-
 /*
   mem[obj] returns a bytes object holding the data for one element if
            obj fully indexes the memory view or another memory-view object
@@ -579,7 +536,37 @@ memory_subscript(PyMemoryViewObject *self, PyObject *key)
         result = PyNumber_AsSsize_t(key, NULL);
         if (result == -1 && PyErr_Occurred())
                 return NULL;
-        return memory_item(self, result);
+        if (view->ndim == 1) {
+            /* Return a bytes object */
+            char *ptr;
+            ptr = (char *)view->buf;
+            if (result < 0) {
+                result += get_shape0(view);
+            }
+            if ((result < 0) || (result >= get_shape0(view))) {
+                PyErr_SetString(PyExc_IndexError,
+                                "index out of bounds");
+                return NULL;
+            }
+            if (view->strides == NULL)
+                ptr += view->itemsize * result;
+            else
+                ptr += view->strides[0] * result;
+            if (view->suboffsets != NULL &&
+                view->suboffsets[0] >= 0) {
+                ptr = *((char **)ptr) + view->suboffsets[0];
+            }
+            return PyBytes_FromStringAndSize(ptr, view->itemsize);
+        }
+        else {
+            /* Return a new memory-view object */
+            Py_buffer newview;
+            memset(&newview, 0, sizeof(newview));
+            /* XXX:  This needs to be fixed so it
+                         actually returns a sub-view
+            */
+            return PyMemoryView_FromBuffer(&newview);
+        }
     }
     else if (PySlice_Check(key)) {
         Py_ssize_t start, stop, step, slicelength;
@@ -626,7 +613,7 @@ memory_subscript(PyMemoryViewObject *self, PyObject *key)
 static int
 memory_ass_sub(PyMemoryViewObject *self, PyObject *key, PyObject *value)
 {
-    Py_ssize_t start, len, bytelen;
+    Py_ssize_t start, len, bytelen, i;
     Py_buffer srcview;
     Py_buffer *view = &(self->view);
     char *srcbuf, *destbuf;
@@ -636,11 +623,6 @@ memory_ass_sub(PyMemoryViewObject *self, PyObject *key, PyObject *value)
             "cannot modify read-only memory");
         return -1;
     }
-    if (value == NULL) {
-        PyErr_SetString(PyExc_TypeError,
-                        "cannot delete memory");
-        return -1;
-    }
     if (view->ndim != 1) {
         PyErr_SetNone(PyExc_NotImplementedError);
         return -1;
@@ -701,8 +683,16 @@ memory_ass_sub(PyMemoryViewObject *self, PyObject *key, PyObject *value)
     if (destbuf + bytelen < srcbuf || srcbuf + bytelen < destbuf)
         /* No overlapping */
         memcpy(destbuf, srcbuf, bytelen);
-    else
-        memmove(destbuf, srcbuf, bytelen);
+    else if (destbuf < srcbuf) {
+        /* Copy in ascending order */
+        for (i = 0; i < bytelen; i++)
+            destbuf[i] = srcbuf[i];
+    }
+    else {
+        /* Copy in descencing order */
+        for (i = bytelen - 1; i >= 0; i--)
+            destbuf[i] = srcbuf[i];
+    }
 
     PyBuffer_Release(&srcview);
     return 0;
@@ -781,19 +771,10 @@ static PyMappingMethods memory_as_mapping = {
     (objobjargproc)memory_ass_sub,        /* mp_ass_subscript */
 };
 
-static PySequenceMethods memory_as_sequence = {
-	0,                                  /* sq_length */
-	0,                                  /* sq_concat */
-	0,                                  /* sq_repeat */
-	(ssizeargfunc)memory_item,          /* sq_item */
-};
 
 /* Buffer methods */
+
 static PyBufferProcs memory_as_buffer = {
-    0,                                    /* bf_getreadbuffer */
-    0,                                    /* bf_getwritebuffer */
-    0,                                    /* bf_getsegcount */
-    0,                                    /* bf_getcharbuffer */
     (getbufferproc)memory_getbuf,         /* bf_getbuffer */
     (releasebufferproc)memory_releasebuf, /* bf_releasebuffer */
 };
@@ -808,10 +789,10 @@ PyTypeObject PyMemoryView_Type = {
     0,                                        /* tp_print */
     0,                                        /* tp_getattr */
     0,                                        /* tp_setattr */
-    0,                                        /* tp_compare */
+    0,                                        /* tp_reserved */
     (reprfunc)memory_repr,                    /* tp_repr */
     0,                                        /* tp_as_number */
-    &memory_as_sequence,                      /* tp_as_sequence */
+    0,                                        /* tp_as_sequence */
     &memory_as_mapping,                       /* tp_as_mapping */
     0,                                        /* tp_hash */
     0,                                        /* tp_call */
@@ -819,8 +800,7 @@ PyTypeObject PyMemoryView_Type = {
     PyObject_GenericGetAttr,                  /* tp_getattro */
     0,                                        /* tp_setattro */
     &memory_as_buffer,                        /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC |
-        Py_TPFLAGS_HAVE_NEWBUFFER,            /* tp_flags */
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,  /* tp_flags */
     memory_doc,                               /* tp_doc */
     (traverseproc)memory_traverse,            /* tp_traverse */
     (inquiry)memory_clear,                    /* tp_clear */
