@@ -11,7 +11,7 @@
    * Python always adds an empty entry at the start, which corresponds
      to the current directory.
 
-   * If the PYTHONPATH env. var. exists, it's entries are added next.
+   * If the PYTHONPATH env. var. exists, its entries are added next.
 
    * We look in the registry for "application paths" - that is, sub-keys
      under the main PythonPath registry key.  These are added next (the
@@ -25,7 +25,7 @@
      is set, we believe it.  Otherwise, we use the path of our host .EXE's
      to try and locate our "landmark" (lib\\os.py) and deduce our home.
      - If we DO have a Python Home: The relevant sub-directories (Lib, 
-       plat-win, lib-tk, etc) are based on the Python Home
+       plat-win, etc) are based on the Python Home
      - If we DO NOT have a Python Home, the core Python Path is
        loaded from the registry.  This is the main PythonPath key, 
        and both HKLM and HKCU are combined to form the path)
@@ -56,14 +56,20 @@
 
 #include "Python.h"
 #include "osdefs.h"
+#include <wchar.h>
 
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
 #include <windows.h>
-#include <tchar.h>
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif /* HAVE_SYS_TYPES_H */
+
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif /* HAVE_SYS_STAT_H */
+
 #include <string.h>
 
 /* Search in some common locations for the associated Python libraries.
@@ -76,16 +82,17 @@
  */
 
 #ifndef LANDMARK
-#define LANDMARK "lib\\os.py"
+#define LANDMARK L"lib\\os.py"
 #endif
 
-static char prefix[MAXPATHLEN+1];
-static char progpath[MAXPATHLEN+1];
-static char *module_search_path = NULL;
+static wchar_t prefix[MAXPATHLEN+1];
+static wchar_t progpath[MAXPATHLEN+1];
+static wchar_t dllpath[MAXPATHLEN+1];
+static wchar_t *module_search_path = NULL;
 
 
 static int
-is_sep(char ch)	/* determine if "ch" is a separator character */
+is_sep(wchar_t ch)	/* determine if "ch" is a separator character */
 {
 #ifdef ALTSEP
 	return ch == SEP || ch == ALTSEP;
@@ -98,9 +105,9 @@ is_sep(char ch)	/* determine if "ch" is a separator character */
    beyond existing terminator.
 */
 static void
-reduce(char *dir)
+reduce(wchar_t *dir)
 {
-	size_t i = strlen(dir);
+	size_t i = wcslen(dir);
 	while (i > 0 && !is_sep(dir[i]))
 		--i;
 	dir[i] = '\0';
@@ -108,46 +115,55 @@ reduce(char *dir)
 	
 
 static int
-exists(char *filename)
+exists(wchar_t *filename)
 {
-	struct stat buf;
-	return stat(filename, &buf) == 0;
+	return GetFileAttributesW(filename) != 0xFFFFFFFF;
 }
 
 /* Assumes 'filename' MAXPATHLEN+1 bytes long - 
    may extend 'filename' by one character.
 */
 static int
-ismodule(char *filename)	/* Is module -- check for .pyc/.pyo too */
+ismodule(wchar_t *filename)	/* Is module -- check for .pyc/.pyo too */
 {
 	if (exists(filename))
 		return 1;
 
 	/* Check for the compiled version of prefix. */
-	if (strlen(filename) < MAXPATHLEN) {
-		strcat(filename, Py_OptimizeFlag ? "o" : "c");
+	if (wcslen(filename) < MAXPATHLEN) {
+		wcscat(filename, Py_OptimizeFlag ? L"o" : L"c");
 		if (exists(filename))
 			return 1;
 	}
 	return 0;
 }
 
-/* guarantees buffer will never overflow MAXPATHLEN+1 bytes */
+/* Add a path component, by appending stuff to buffer.
+   buffer must have at least MAXPATHLEN + 1 bytes allocated, and contain a
+   NUL-terminated string with no more than MAXPATHLEN characters (not counting
+   the trailing NUL).  It's a fatal error if it contains a string longer than
+   that (callers must be careful!).  If these requirements are met, it's
+   guaranteed that buffer will still be a NUL-terminated string with no more
+   than MAXPATHLEN characters at exit.  If stuff is too long, only as much of
+   stuff as fits will be appended.
+*/
 static void
-join(char *buffer, char *stuff)
+join(wchar_t *buffer, wchar_t *stuff)
 {
 	size_t n, k;
 	if (is_sep(stuff[0]))
 		n = 0;
 	else {
-		n = strlen(buffer);
+		n = wcslen(buffer);
 		if (n > 0 && !is_sep(buffer[n-1]) && n < MAXPATHLEN)
 			buffer[n++] = SEP;
 	}
-	k = strlen(stuff);
+	if (n > MAXPATHLEN)
+		Py_FatalError("buffer overflow in getpathp.c's joinpath()");
+	k = wcslen(stuff);
 	if (n + k > MAXPATHLEN)
 		k = MAXPATHLEN - n;
-	strncpy(buffer+n, stuff, k);
+	wcsncpy(buffer+n, stuff, k);
 	buffer[n+k] = '\0';
 }
 
@@ -156,11 +172,12 @@ join(char *buffer, char *stuff)
    'landmark' can not overflow prefix if too long.
 */
 static int
-gotlandmark(char *landmark)
+gotlandmark(wchar_t *landmark)
 {
-	int n, ok;
+	int ok;
+	Py_ssize_t n;
 
-	n = strlen(prefix);
+	n = wcslen(prefix);
 	join(prefix, landmark);
 	ok = ismodule(prefix);
 	prefix[n] = '\0';
@@ -170,10 +187,10 @@ gotlandmark(char *landmark)
 /* assumes argv0_path is MAXPATHLEN+1 bytes long, already \0 term'd. 
    assumption provided by only caller, calculate_path() */
 static int
-search_for_prefix(char *argv0_path, char *landmark)
+search_for_prefix(wchar_t *argv0_path, wchar_t *landmark)
 {
 	/* Search from argv0_path, until landmark is found */
-	strcpy(prefix, argv0_path);
+	wcscpy(prefix, argv0_path);
 	do {
 		if (gotlandmark(landmark))
 			return 1;
@@ -182,7 +199,8 @@ search_for_prefix(char *argv0_path, char *landmark)
 	return 0;
 }
 
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
+#ifdef Py_ENABLE_SHARED
 
 /* a string loaded from the DLL at startup.*/
 extern const char *PyWin_DLLVersionString;
@@ -201,39 +219,39 @@ extern const char *PyWin_DLLVersionString;
    in advance.  It could be simplied now Win16/Win32s is dead!
 */
 
-static char *
+static wchar_t *
 getpythonregpath(HKEY keyBase, int skipcore)
 {
 	HKEY newKey = 0;
 	DWORD dataSize = 0;
 	DWORD numKeys = 0;
 	LONG rc;
-	char *retval = NULL;
-	TCHAR *dataBuf = NULL;
-	static const TCHAR keyPrefix[] = _T("Software\\Python\\PythonCore\\");
-	static const TCHAR keySuffix[] = _T("\\PythonPath");
+	wchar_t *retval = NULL;
+	WCHAR *dataBuf = NULL;
+	static const WCHAR keyPrefix[] = L"Software\\Python\\PythonCore\\";
+	static const WCHAR keySuffix[] = L"\\PythonPath";
 	size_t versionLen;
 	DWORD index;
-	TCHAR *keyBuf = NULL;
-	TCHAR *keyBufPtr;
-	TCHAR **ppPaths = NULL;
-
+	WCHAR *keyBuf = NULL;
+	WCHAR *keyBufPtr;
+	WCHAR **ppPaths = NULL;
+	
 	/* Tried to use sysget("winver") but here is too early :-( */
-	versionLen = _tcslen(PyWin_DLLVersionString);
+	versionLen = strlen(PyWin_DLLVersionString);
 	/* Space for all the chars, plus one \0 */
 	keyBuf = keyBufPtr = malloc(sizeof(keyPrefix) + 
-		                    sizeof(TCHAR)*(versionLen-1) + 
+		                    sizeof(WCHAR)*(versionLen-1) + 
 				    sizeof(keySuffix));
 	if (keyBuf==NULL) goto done;
 
-	memcpy(keyBufPtr, keyPrefix, sizeof(keyPrefix)-sizeof(TCHAR));
-	keyBufPtr += sizeof(keyPrefix)/sizeof(TCHAR) - 1;
-	memcpy(keyBufPtr, PyWin_DLLVersionString, versionLen * sizeof(TCHAR));
+	memcpy(keyBufPtr, keyPrefix, sizeof(keyPrefix)-sizeof(WCHAR));
+	keyBufPtr += sizeof(keyPrefix)/sizeof(WCHAR) - 1;
+	mbstowcs(keyBufPtr, PyWin_DLLVersionString, versionLen);
 	keyBufPtr += versionLen;
 	/* NULL comes with this one! */
 	memcpy(keyBufPtr, keySuffix, sizeof(keySuffix));
 	/* Open the root Python key */
-	rc=RegOpenKeyEx(keyBase,
+	rc=RegOpenKeyExW(keyBase,
 	                keyBuf, /* subkey */
 	                0, /* reserved */
 	                KEY_READ,
@@ -247,31 +265,31 @@ getpythonregpath(HKEY keyBase, int skipcore)
 	/* Allocate a temp array of char buffers, so we only need to loop 
 	   reading the registry once
 	*/
-	ppPaths = malloc( sizeof(TCHAR *) * numKeys );
+	ppPaths = malloc( sizeof(WCHAR *) * numKeys );
 	if (ppPaths==NULL) goto done;
-	memset(ppPaths, 0, sizeof(TCHAR *) * numKeys);
+	memset(ppPaths, 0, sizeof(WCHAR *) * numKeys);
 	/* Loop over all subkeys, allocating a temp sub-buffer. */
 	for(index=0;index<numKeys;index++) {
-		TCHAR keyBuf[MAX_PATH+1];
+		WCHAR keyBuf[MAX_PATH+1];
 		HKEY subKey = 0;
 		DWORD reqdSize = MAX_PATH+1;
 		/* Get the sub-key name */
-		DWORD rc = RegEnumKeyEx(newKey, index, keyBuf, &reqdSize,
-		                        NULL, NULL, NULL, NULL );
+		DWORD rc = RegEnumKeyExW(newKey, index, keyBuf, &reqdSize,
+					 NULL, NULL, NULL, NULL );
 		if (rc!=ERROR_SUCCESS) goto done;
 		/* Open the sub-key */
-		rc=RegOpenKeyEx(newKey,
+		rc=RegOpenKeyExW(newKey,
 						keyBuf, /* subkey */
 						0, /* reserved */
 						KEY_READ,
 						&subKey);
 		if (rc!=ERROR_SUCCESS) goto done;
 		/* Find the value of the buffer size, malloc, then read it */
-		RegQueryValueEx(subKey, NULL, 0, NULL, NULL, &reqdSize);
+		RegQueryValueExW(subKey, NULL, 0, NULL, NULL, &reqdSize);
 		if (reqdSize) {
 			ppPaths[index] = malloc(reqdSize);
 			if (ppPaths[index]) {
-				RegQueryValueEx(subKey, NULL, 0, NULL, 
+				RegQueryValueExW(subKey, NULL, 0, NULL, 
 				                (LPBYTE)ppPaths[index], 
 				                &reqdSize);
 				dataSize += reqdSize + 1; /* 1 for the ";" */
@@ -279,22 +297,27 @@ getpythonregpath(HKEY keyBase, int skipcore)
 		}
 		RegCloseKey(subKey);
 	}
+
+	/* return null if no path to return */
+	if (dataSize == 0) goto done;
+
 	/* original datasize from RegQueryInfo doesn't include the \0 */
-	dataBuf = malloc((dataSize+1) * sizeof(TCHAR));
+	dataBuf = malloc((dataSize+1) * sizeof(WCHAR));
 	if (dataBuf) {
-		TCHAR *szCur = dataBuf;
+		WCHAR *szCur = dataBuf;
 		DWORD reqdSize = dataSize;
 		/* Copy our collected strings */
 		for (index=0;index<numKeys;index++) {
 			if (index > 0) {
-				*(szCur++) = _T(';');
+				*(szCur++) = L';';
 				dataSize--;
 			}
 			if (ppPaths[index]) {
-				int len = _tcslen(ppPaths[index]);
-				_tcsncpy(szCur, ppPaths[index], len);
+				Py_ssize_t len = wcslen(ppPaths[index]);
+				wcsncpy(szCur, ppPaths[index], len);
 				szCur += len;
-				dataSize -= len;
+				assert(dataSize > (DWORD)len);
+				dataSize -= (DWORD)len;
 			}
 		}
 		if (skipcore)
@@ -302,30 +325,17 @@ getpythonregpath(HKEY keyBase, int skipcore)
 		else {
 			/* If we have no values, we dont need a ';' */
 			if (numKeys) {
-				*(szCur++) = _T(';');
+				*(szCur++) = L';';
 				dataSize--;
 			}
 			/* Now append the core path entries - 
 			   this will include the NULL 
 			*/
-			rc = RegQueryValueEx(newKey, NULL, 0, NULL, 
-			                     (LPBYTE)szCur, &dataSize);
+			rc = RegQueryValueExW(newKey, NULL, 0, NULL, 
+					      (LPBYTE)szCur, &dataSize);
 		}
-		/* And set the result - caller must free 
-		   If MBCS, it is fine as is.  If Unicode, allocate new
-		   buffer and convert.
-		*/
-#ifdef UNICODE
-		retval = (char *)malloc(reqdSize+1);
-		if (retval)
-			WideCharToMultiByte(CP_ACP, 0, 
-					dataBuf, -1, /* source */ 
-					retval, dataSize+1, /* dest */
-					NULL, NULL);
-		free(dataBuf);
-#else
+		/* And set the result - caller must free */
 		retval = dataBuf;
-#endif
 	}
 done:
 	/* Loop freeing my temp buffers */
@@ -340,38 +350,31 @@ done:
 		free(keyBuf);
 	return retval;
 }
-#endif /* MS_WIN32 */
+#endif /* Py_ENABLE_SHARED */
+#endif /* MS_WINDOWS */
 
 static void
 get_progpath(void)
 {
-	extern char *Py_GetProgramName(void);
-	char *path = getenv("PATH");
-	char *prog = Py_GetProgramName();
+	extern wchar_t *Py_GetProgramName(void);
+	wchar_t *path = _wgetenv(L"PATH");
+	wchar_t *prog = Py_GetProgramName();
 
-#ifdef MS_WIN32
-#ifdef UNICODE
-	WCHAR wprogpath[MAXPATHLEN+1];
-	/* Windows documents that GetModuleFileName() will "truncate",
-	   but makes no mention of the null terminator.  Play it safe.
-	   PLUS Windows itself defines MAX_PATH as the same, but anyway...
-	*/
-	wprogpath[MAXPATHLEN]=_T('\0')';
-	if (GetModuleFileName(NULL, wprogpath, MAXPATHLEN)) {
-		WideCharToMultiByte(CP_ACP, 0, 
-		                    wprogpath, -1, 
-		                    progpath, MAXPATHLEN+1, 
-		                    NULL, NULL);
-		return;
-	}
-#else
+#ifdef MS_WINDOWS
+#ifdef Py_ENABLE_SHARED
+	extern HANDLE PyWin_DLLhModule;
 	/* static init of progpath ensures final char remains \0 */
-	if (GetModuleFileName(NULL, progpath, MAXPATHLEN))
-		return;
+	if (PyWin_DLLhModule)
+		if (!GetModuleFileNameW(PyWin_DLLhModule, dllpath, MAXPATHLEN))
+			dllpath[0] = 0;
+#else
+	dllpath[0] = 0;
 #endif
+	if (GetModuleFileNameW(NULL, progpath, MAXPATHLEN))
+		return;
 #endif
 	if (prog == NULL || *prog == '\0')
-		prog = "python";
+		prog = L"python";
 
 	/* If there is no slash in the argv0 path, then we have to
 	 * assume python is on the user's $PATH, since there's no
@@ -379,24 +382,24 @@ get_progpath(void)
 	 * $PATH isn't exported, you lose.
 	 */
 #ifdef ALTSEP
-	if (strchr(prog, SEP) || strchr(prog, ALTSEP))
+	if (wcschr(prog, SEP) || wcschr(prog, ALTSEP))
 #else
-	if (strchr(prog, SEP))
+	if (wcschr(prog, SEP))
 #endif
-		strncpy(progpath, prog, MAXPATHLEN);
+		wcsncpy(progpath, prog, MAXPATHLEN);
 	else if (path) {
 		while (1) {
-			char *delim = strchr(path, DELIM);
+			wchar_t *delim = wcschr(path, DELIM);
 
 			if (delim) {
 				size_t len = delim - path;
 				/* ensure we can't overwrite buffer */
 				len = min(MAXPATHLEN,len);
-				strncpy(progpath, path, len);
+				wcsncpy(progpath, path, len);
 				*(progpath + len) = '\0';
 			}
 			else
-				strncpy(progpath, path, MAXPATHLEN);
+				wcsncpy(progpath, path, MAXPATHLEN);
 
 			/* join() is safe for MAXPATHLEN+1 size buffer */
 			join(progpath, prog);
@@ -417,21 +420,31 @@ get_progpath(void)
 static void
 calculate_path(void)
 {
-	char argv0_path[MAXPATHLEN+1];
-	char *buf;
+	wchar_t argv0_path[MAXPATHLEN+1];
+	wchar_t *buf;
 	size_t bufsz;
-	char *pythonhome = Py_GetPythonHome();
-	char *envpath = Py_GETENV("PYTHONPATH");
+	wchar_t *pythonhome = Py_GetPythonHome();
+	char *_envpath = Py_GETENV("PYTHONPATH");
+	wchar_t wenvpath[MAXPATHLEN+1];
+	wchar_t *envpath = NULL;
 
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
 	int skiphome, skipdefault;
-	char *machinepath = NULL;
-	char *userpath = NULL;
+	wchar_t *machinepath = NULL;
+	wchar_t *userpath = NULL;
+	wchar_t zip_path[MAXPATHLEN+1];
+	size_t len;
 #endif
+	if (_envpath) {
+		size_t r = mbstowcs(wenvpath, _envpath, MAXPATHLEN+1);
+		envpath = wenvpath;
+		if (r == (size_t)-1 || r >= MAXPATHLEN)
+			envpath = NULL;
+	}
 
 	get_progpath();
 	/* progpath guaranteed \0 terminated in MAXPATH+1 bytes. */
-	strcpy(argv0_path, progpath);
+	wcscpy(argv0_path, progpath);
 	reduce(argv0_path);
 	if (pythonhome == NULL || *pythonhome == '\0') {
 		if (search_for_prefix(argv0_path, LANDMARK))
@@ -440,16 +453,34 @@ calculate_path(void)
 			pythonhome = NULL;
 	}
 	else
-		strncpy(prefix, pythonhome, MAXPATHLEN);
+		wcsncpy(prefix, pythonhome, MAXPATHLEN);
 
 	if (envpath && *envpath == '\0')
 		envpath = NULL;
 
 
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
+	/* Calculate zip archive path */
+	if (dllpath[0])		/* use name of python DLL */
+		wcsncpy(zip_path, dllpath, MAXPATHLEN);
+	else			/* use name of executable program */
+		wcsncpy(zip_path, progpath, MAXPATHLEN);
+	zip_path[MAXPATHLEN] = '\0';
+	len = wcslen(zip_path);
+	if (len > 4) {
+		zip_path[len-3] = 'z';	/* change ending to "zip" */
+		zip_path[len-2] = 'i';
+		zip_path[len-1] = 'p';
+	}
+	else {
+		zip_path[0] = 0;
+	}
+ 
 	skiphome = pythonhome==NULL ? 0 : 1;
+#ifdef Py_ENABLE_SHARED
 	machinepath = getpythonregpath(HKEY_LOCAL_MACHINE, skiphome);
 	userpath = getpythonregpath(HKEY_CURRENT_USER, skiphome);
+#endif
 	/* We only use the default relative PYTHONPATH if we havent
 	   anything better to use! */
 	skipdefault = envpath!=NULL || pythonhome!=NULL || \
@@ -458,40 +489,42 @@ calculate_path(void)
 
 	/* We need to construct a path from the following parts.
 	   (1) the PYTHONPATH environment variable, if set;
-	   (2) for Win32, the machinepath and userpath, if set;
-	   (3) the PYTHONPATH config macro, with the leading "."
+	   (2) for Win32, the zip archive file path;
+	   (3) for Win32, the machinepath and userpath, if set;
+	   (4) the PYTHONPATH config macro, with the leading "."
 	       of each component replaced with pythonhome, if set;
-	   (4) the directory containing the executable (argv0_path).
-	   The length calculation calculates #3 first.
+	   (5) the directory containing the executable (argv0_path).
+	   The length calculation calculates #4 first.
 	   Extra rules:
-	   - If PYTHONHOME is set (in any way) item (2) is ignored.
-	   - If registry values are used, (3) and (4) are ignored.
+	   - If PYTHONHOME is set (in any way) item (3) is ignored.
+	   - If registry values are used, (4) and (5) are ignored.
 	*/
 
 	/* Calculate size of return buffer */
 	if (pythonhome != NULL) {
-		char *p;
+		wchar_t *p;
 		bufsz = 1;	
 		for (p = PYTHONPATH; *p; p++) {
 			if (*p == DELIM)
 				bufsz++; /* number of DELIM plus one */
 		}
-		bufsz *= strlen(pythonhome);
+		bufsz *= wcslen(pythonhome);
 	}
 	else
 		bufsz = 0;
-	bufsz += strlen(PYTHONPATH) + 1;
-	bufsz += strlen(argv0_path) + 1;
-#ifdef MS_WIN32
+	bufsz += wcslen(PYTHONPATH) + 1;
+	bufsz += wcslen(argv0_path) + 1;
+#ifdef MS_WINDOWS
 	if (userpath)
-		bufsz += strlen(userpath) + 1;
+		bufsz += wcslen(userpath) + 1;
 	if (machinepath)
-		bufsz += strlen(machinepath) + 1;
+		bufsz += wcslen(machinepath) + 1;
+	bufsz += wcslen(zip_path) + 1;
 #endif
 	if (envpath != NULL)
-		bufsz += strlen(envpath) + 1;
+		bufsz += wcslen(envpath) + 1;
 
-	module_search_path = buf = malloc(bufsz);
+	module_search_path = buf = malloc(bufsz*sizeof(wchar_t));
 	if (buf == NULL) {
 		/* We can't exit, so print a warning and limp along */
 		fprintf(stderr, "Can't malloc dynamic PYTHONPATH.\n");
@@ -503,62 +536,67 @@ calculate_path(void)
 			fprintf(stderr, "Using default static path.\n");
 			module_search_path = PYTHONPATH;
 		}
-#ifdef MS_WIN32
+#ifdef MS_WINDOWS
 		if (machinepath)
 			free(machinepath);
 		if (userpath)
 			free(userpath);
-#endif /* MS_WIN32 */
+#endif /* MS_WINDOWS */
 		return;
 	}
 
 	if (envpath) {
-		strcpy(buf, envpath);
-		buf = strchr(buf, '\0');
+		wcscpy(buf, envpath);
+		buf = wcschr(buf, L'\0');
+		*buf++ = DELIM;
+	}
+#ifdef MS_WINDOWS
+	if (zip_path[0]) {
+		wcscpy(buf, zip_path);
+		buf = wcschr(buf, L'\0');
 		*buf++ = DELIM;
 	}
-#ifdef MS_WIN32
 	if (userpath) {
-		strcpy(buf, userpath);
-		buf = strchr(buf, '\0');
+		wcscpy(buf, userpath);
+		buf = wcschr(buf, L'\0');
 		*buf++ = DELIM;
 		free(userpath);
 	}
 	if (machinepath) {
-		strcpy(buf, machinepath);
-		buf = strchr(buf, '\0');
+		wcscpy(buf, machinepath);
+		buf = wcschr(buf, L'\0');
 		*buf++ = DELIM;
 		free(machinepath);
 	}
 	if (pythonhome == NULL) {
 		if (!skipdefault) {
-			strcpy(buf, PYTHONPATH);
-			buf = strchr(buf, '\0');
+			wcscpy(buf, PYTHONPATH);
+			buf = wcschr(buf, L'\0');
 		}
 	}
 #else
 	if (pythonhome == NULL) {
-		strcpy(buf, PYTHONPATH);
-		buf = strchr(buf, '\0');
+		wcscpy(buf, PYTHONPATH);
+		buf = wcschr(buf, L'\0');
 	}
-#endif /* MS_WIN32 */
+#endif /* MS_WINDOWS */
 	else {
-		char *p = PYTHONPATH;
-		char *q;
+		wchar_t *p = PYTHONPATH;
+		wchar_t *q;
 		size_t n;
 		for (;;) {
-			q = strchr(p, DELIM);
+			q = wcschr(p, DELIM);
 			if (q == NULL)
-				n = strlen(p);
+				n = wcslen(p);
 			else
 				n = q-p;
 			if (p[0] == '.' && is_sep(p[1])) {
-				strcpy(buf, pythonhome);
-				buf = strchr(buf, '\0');
+				wcscpy(buf, pythonhome);
+				buf = wcschr(buf, L'\0');
 				p++;
 				n--;
 			}
-			strncpy(buf, p, n);
+			wcsncpy(buf, p, n);
 			buf += n;
 			if (q == NULL)
 				break;
@@ -568,10 +606,10 @@ calculate_path(void)
 	}
 	if (argv0_path) {
 		*buf++ = DELIM;
-		strcpy(buf, argv0_path);
-		buf = strchr(buf, '\0');
+		wcscpy(buf, argv0_path);
+		buf = wcschr(buf, L'\0');
 	}
-	*buf = '\0';
+	*buf = L'\0';
 	/* Now to pull one last hack/trick.  If sys.prefix is
 	   empty, then try and find it somewhere on the paths
 	   we calculated.  We scan backwards, as our general policy
@@ -580,21 +618,21 @@ calculate_path(void)
 	   on the path, and that our 'prefix' directory is
 	   the parent of that.
 	*/
-	if (*prefix=='\0') {
-		char lookBuf[MAXPATHLEN+1];
-		char *look = buf - 1; /* 'buf' is at the end of the buffer */
+	if (*prefix==L'\0') {
+		wchar_t lookBuf[MAXPATHLEN+1];
+		wchar_t *look = buf - 1; /* 'buf' is at the end of the buffer */
 		while (1) {
-			int nchars;
-			char *lookEnd = look;
+			Py_ssize_t nchars;
+			wchar_t *lookEnd = look;
 			/* 'look' will end up one character before the
 			   start of the path in question - even if this
 			   is one character before the start of the buffer
 			*/
-			while (*look != DELIM && look >= module_search_path)
+			while (look >= module_search_path && *look != DELIM)
 				look--;
 			nchars = lookEnd-look;
-			strncpy(lookBuf, look+1, nchars);
-			lookBuf[nchars] = '\0';
+			wcsncpy(lookBuf, look+1, nchars);
+			lookBuf[nchars] = L'\0';
 			/* Up one level to the parent */
 			reduce(lookBuf);
 			if (search_for_prefix(lookBuf, LANDMARK)) {
@@ -611,7 +649,7 @@ calculate_path(void)
 
 /* External interface */
 
-char *
+wchar_t *
 Py_GetPath(void)
 {
 	if (!module_search_path)
@@ -619,7 +657,7 @@ Py_GetPath(void)
 	return module_search_path;
 }
 
-char *
+wchar_t *
 Py_GetPrefix(void)
 {
 	if (!module_search_path)
@@ -627,13 +665,13 @@ Py_GetPrefix(void)
 	return prefix;
 }
 
-char *
+wchar_t *
 Py_GetExecPrefix(void)
 {
 	return Py_GetPrefix();
 }
 
-char *
+wchar_t *
 Py_GetProgramFullPath(void)
 {
 	if (!module_search_path)
